@@ -36,73 +36,25 @@
 
 struct _GVirDomainDiskPrivate
 {
-    gchar *path;
+    gboolean unused;
 };
 
 G_DEFINE_TYPE(GVirDomainDisk, gvir_domain_disk, GVIR_TYPE_DOMAIN_DEVICE);
 
-enum {
-    PROP_0,
-    PROP_PATH,
-};
-
 #define GVIR_DOMAIN_DISK_ERROR gvir_domain_disk_error_quark()
 
-
 static GQuark
 gvir_domain_disk_error_quark(void)
 {
     return g_quark_from_static_string("gvir-domain-disk");
 }
 
-static void gvir_domain_disk_get_property(GObject *object,
-                                          guint prop_id,
-                                          GValue *value,
-                                          GParamSpec *pspec)
-{
-    GVirDomainDisk *self = GVIR_DOMAIN_DISK(object);
-    GVirDomainDiskPrivate *priv = self->priv;
-
-    switch (prop_id) {
-    case PROP_PATH:
-        g_value_set_string(value, priv->path);
-        break;
-
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-    }
-}
-
-
-static void gvir_domain_disk_set_property(GObject *object,
-                                          guint prop_id,
-                                          const GValue *value,
-                                          GParamSpec *pspec)
-{
-    GVirDomainDisk *self = GVIR_DOMAIN_DISK(object);
-    GVirDomainDiskPrivate *priv = self->priv;
-
-    switch (prop_id) {
-    case PROP_PATH:
-        g_free(priv->path);
-        priv->path = g_value_dup_string(value);
-        break;
-
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-    }
-}
-
-
 static void gvir_domain_disk_finalize(GObject *object)
 {
     GVirDomainDisk *self = GVIR_DOMAIN_DISK(object);
-    GVirDomainDiskPrivate *priv = self->priv;
 
     g_debug("Finalize GVirDomainDisk=%p", self);
 
-    g_free(priv->path);
-
     G_OBJECT_CLASS(gvir_domain_disk_parent_class)->finalize(object);
 }
 
@@ -111,19 +63,6 @@ static void gvir_domain_disk_class_init(GVirDomainDiskClass *klass)
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
     object_class->finalize = gvir_domain_disk_finalize;
-    object_class->get_property = gvir_domain_disk_get_property;
-    object_class->set_property = gvir_domain_disk_set_property;
-
-    g_object_class_install_property(object_class,
-                                    PROP_PATH,
-                                    g_param_spec_string("path",
-                                                        "Path",
-                                                        "The disk path",
-                                                        NULL,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
-
     g_type_class_add_private(klass, sizeof(GVirDomainDiskPrivate));
 }
 
@@ -151,6 +90,19 @@ gvir_domain_disk_stats_free(GVirDomainDiskStats *stats)
 G_DEFINE_BOXED_TYPE(GVirDomainDiskStats, gvir_domain_disk_stats,
                     gvir_domain_disk_stats_copy, gvir_domain_disk_stats_free)
 
+static gchar *gvir_domain_disk_get_path(GVirDomainDisk *self)
+{
+    GVirConfigDomainDevice *config;
+    gchar *path;
+
+    config = gvir_domain_device_get_config(GVIR_DOMAIN_DEVICE(self));
+    path = gvir_config_domain_disk_get_target_dev(GVIR_CONFIG_DOMAIN_DISK(config));
+
+    g_object_unref(config);
+
+    return path;
+}
+
 /**
  * gvir_domain_disk_get_stats:
  * @self: the domain disk
@@ -166,15 +118,15 @@ GVirDomainDiskStats *gvir_domain_disk_get_stats(GVirDomainDisk *self, GError **e
 {
     GVirDomainDiskStats *ret = NULL;
     virDomainBlockStatsStruct stats;
-    GVirDomainDiskPrivate *priv;
     virDomainPtr handle;
+    gchar *path;
 
     g_return_val_if_fail(GVIR_IS_DOMAIN_DISK(self), NULL);
 
-    priv = self->priv;
     handle = gvir_domain_device_get_domain_handle(GVIR_DOMAIN_DEVICE(self));
+    path = gvir_domain_disk_get_path (self);
 
-    if (virDomainBlockStats(handle, priv->path, &stats, sizeof (stats)) < 0) {
+    if (virDomainBlockStats(handle, path, &stats, sizeof (stats)) < 0) {
         gvir_set_error_literal(err, GVIR_DOMAIN_DISK_ERROR,
                                0,
                                "Unable to get domain disk stats");
@@ -190,5 +142,47 @@ GVirDomainDiskStats *gvir_domain_disk_get_stats(GVirDomainDisk *self, GError **e
 
 end:
     virDomainFree(handle);
+    g_free(path);
+    return ret;
+}
+
+/**
+ * gvir_domain_disk_resize:
+ * @self: the domain disk
+ * @size: new size of the block image in kilobytes
+ * @flags: flags, currently unused. Pass '0'.
+ * @err: placeholder for an error, or NULL
+ *
+ * This function resize the disk of a running domain.
+ *
+ * Returns: TRUE if size was successfully changed, FALSE otherwise.
+ **/
+gboolean gvir_domain_disk_resize(GVirDomainDisk *self,
+                                 guint64 size,
+                                 guint flags,
+                                 GError **err)
+{
+    gboolean ret = FALSE;
+    virDomainPtr handle;
+    gchar *path;
+
+    g_return_val_if_fail(GVIR_IS_DOMAIN_DISK(self), FALSE);
+    g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+
+    handle = gvir_domain_device_get_domain_handle(GVIR_DOMAIN_DEVICE(self));
+    path = gvir_domain_disk_get_path (self);
+
+    if (virDomainBlockResize(handle, path, size, flags) < 0) {
+        gvir_set_error_literal(err, GVIR_DOMAIN_DISK_ERROR,
+                               0,
+                               "Failed to resize domain disk");
+        goto end;
+    }
+
+    ret = TRUE;
+
+end:
+    virDomainFree(handle);
+    g_free(path);
     return ret;
 }
