@@ -30,6 +30,7 @@
 G_BEGIN_DECLS
 
 #include <libvirt-gobject/libvirt-gobject-stream.h>
+#include <libvirt/libvirt.h>
 
 #define GVIR_TYPE_DOMAIN            (gvir_domain_get_type ())
 #define GVIR_DOMAIN(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GVIR_TYPE_DOMAIN, GVirDomain))
@@ -88,6 +89,19 @@ typedef enum {
     GVIR_DOMAIN_START_FORCE_BOOT   = (1 << 3),
 } GVirDomainStartFlags;
 
+/**
+ * GVirDomainDeleteFlags:
+ * @GVIR_DOMAIN_DELETE_NONE: No flags
+ * @GVIR_DOMAIN_DELETE_SAVED_STATE: Also remove associated saved state (if present).
+ * @GVIR_DOMAIN_DELETE_SNAPSHOTS_METADATA: If last use of domain, then also
+ *                                         remove any snapshot metadata.
+ */
+typedef enum {
+    GVIR_DOMAIN_DELETE_NONE               = 0,
+    GVIR_DOMAIN_DELETE_SAVED_STATE        = VIR_DOMAIN_UNDEFINE_MANAGED_SAVE,
+    GVIR_DOMAIN_DELETE_SNAPSHOTS_METADATA = VIR_DOMAIN_UNDEFINE_SNAPSHOTS_METADATA,
+} GVirDomainDeleteFlags;
+
 typedef struct _GVirDomainInfo GVirDomainInfo;
 struct _GVirDomainInfo
 {
@@ -167,6 +181,10 @@ gboolean gvir_domain_save_finish (GVirDomain *dom,
                                   GAsyncResult *result,
                                   GError **err);
 gboolean gvir_domain_get_persistent(GVirDomain *dom);
+gboolean gvir_domain_get_saved(GVirDomain *dom);
+
+GList *gvir_domain_get_devices(GVirDomain *domain,
+                               GError **err);
 
 G_END_DECLS
 
